#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifndef USE_NETLIB
#include <net/ethernet.h>
#include <netinet/in.h>
#ifdef __linux__
#include <netinet/ether.h>
#endif
#include <arpa/inet.h>
#else
#include <netlib.h>
#endif

#include "pktlib.h"
#include "pktbuf.h"

static char *macaddr_s(char macaddr[])
{
  return ether_ntoa((struct ether_addr *)macaddr);
}

static char *ipaddr_s(unsigned int ipaddr)
{
  struct in_addr in;
  in.s_addr = htonl(ipaddr);
  return inet_ntoa(in);
}

int main(int argc, char *argv[])
{
  pktif_t pktif;
  pktbuf_t pktbuf;
  int r = 0, next;
  struct header_param param;

  argc--;
  argv++;

  pktif = pktif_open(argv[0], 0, 0);

  pktbuf = pktbuf_recv(pktif, 0);
  if (pktbuf)
    pktbuf_destroy(pktbuf);

  next = pktif_get_linktype(pktif);

  switch (next) {
  case DLT_EN10MB:
    r = pktbuf_analyze_ethernet(pktbuf, &param);
    break;
  default:
    break;
  }

  if (r > 0) {
    switch (next) {
    case DLT_EN10MB:
      printf("Ethernet:\n");
      printf("\tsrc macaddr: %s\n", macaddr_s(param.ethernet.src_macaddr));
      printf("\tdst macaddr: %s\n", macaddr_s(param.ethernet.dst_macaddr));
      printf("\ttype       : %04x\n", param.ethernet.type);
      next = param.ethernet.type;
      break;
    default:
      next = -1;
      break;
    }
  }

  if (r > 1) {
    switch (next) {
    case ETHERTYPE_IP:
      printf("IP:\n");
      printf("\tid        : %04x\n", param.ip.id);
      printf("\tttl       : %04x\n", param.ip.ttl);
      printf("\tprotocol  : %d\n", param.ip.protocol);
      printf("\tsrc ipaddr: %s\n", ipaddr_s(param.ip.src_ipaddr));
      printf("\tdst ipaddr: %s\n", ipaddr_s(param.ip.dst_ipaddr));
      next = param.ip.protocol;
      break;
    default:
      next = -1;
      break;
    }
  }

  if (r > 2) {
    switch (next) {
    case IPPROTO_UDP:
      printf("UDP:\n");
      printf("\tsrc port: %d\n", param.udp.src_port);
      printf("\tdst port: %d\n", param.udp.dst_port);
      break;
    default:
      break;
    }
  }

  return 0;
}
